INTRODUCTION:

Inline::C - Write Perl subroutines in C.

Inline::C is the Inline Language Support Module (ILSM) for C

Example:

    use Inline C => <<'END';
    SV* JAxH(char* x) {
        return newSVpvf ("Just Another %s Hacker",x);
    }
    END
    
    print JAxH('Inline'), "\n";

When run, this complete program prints:

    Just Another Inline Hacker

-------------------------------------------------------------------------------
INSTALLATION:

This module requires the Inline module with which it is distributed. It also
requires the appropriate C compiler. (Where appropriate means the one referred
to in your Config.pm) 

Inline::C is packaged with Inline.pm. It is not necessary to install Inline::C to use Inline with some other language, like Java. 

-------------------------------------------------------------------------------
INFORMATION:

= For more infor on Inline, see 'perldoc Inline' and 'perldoc Inline-Support'
= For information about Inline::, see 'perldoc Inline::C' and
  'perldoc Inline::C-Cookbook'
= For information on writing your own Inline extension see 'perldoc Inline-API'
= For information about the Perl5 internal C API, see 'perldoc perlapi' or
  try http://www.perldoc.com/perl5.6/pod/perlapi.html

The Inline.pm mailing list is inline@perl.org. 
Send email to inline-subscribe@perl.org to subscribe.

Please send questions and comments to "Brian Ingerson" <INGY@cpan.org>

Copyright (c) 2001, 2002, Brian Ingerson. All Rights Reserved.  
